@@ -10,17 +10,19 @@ use Scalar::Util qw(looks_like_number);
 
 our @EXPORT_OK = qw(ps_parse ps_serialize);
 
+=encoding utf8
+
 =head1 NAME
 
 Struct::Path::PerlStyle - Perl-style syntax frontend for L<Struct::Path|Struct::Path>.
 
 =head1 VERSION
 
-Version 0.42
+Version 0.43
 
 =cut
 
-our $VERSION = '0.42';
+our $VERSION = '0.43';
 
 =head1 SYNOPSIS
 
@@ -42,6 +44,7 @@ Examples:
     "{a}{b,c}"            # b's and c's values
     "{a}{b c}"            # same, space is also a delimiter
     "{a}{'space inside'}" # key must be quoted unless it is a simple word (double quotes supported as well)
+    "{a}{'π'}"            # non ASCII keys also must be quoted
     "{a}{b}[0,1,2,5]"     # 0, 1, 2 and 5 array's items
     "{a}{b}[0..2,5]"      # same, but using ranges
     "{a}{b}[9..0]"        # descending ranges allowed (perl doesn't)
@@ -88,8 +91,11 @@ sub ps_parse($) {
                         $tmp->{keys} = $t->content;
                     } elsif ($t->isa('PPI::Token::Operator') and $t->content eq ',') {
                         next;
-                    } elsif ($t->isa('PPI::Token::Quote')) {
-                        $tmp->{keys} = substr(substr($t->content, 1), 0, -1);
+                    } elsif ($t->isa('PPI::Token::Quote::Single')) {
+                        $tmp->{keys} = $t->literal;
+                    } elsif ($t->isa('PPI::Token::Quote::Double')) {
+                        $tmp->{keys} = $t->string;
+                        $tmp->{keys} =~ s/\\"/"/g;
                     } elsif ($t->isa('PPI::Token::Regexp::Match')) {
                         $tmp->{regs} = substr(substr($t->content, 1), 0, -1); # get rid of slashes
                         $tmp->{regs} = qr($tmp->{regs});
@@ -171,7 +177,19 @@ sub ps_serialize($) {
         } elsif (ref $step eq 'HASH') {
             my @items;
             if (keys %{$step} == 1 and exists $step->{keys} and ref $step->{keys} eq 'ARRAY' or not keys %{$step}) {
-                push @items, map { /^\w+$/ ? $_ : "'$_'" } @{$step->{keys}}; # quote
+                for my $k (@{$step->{keys}}) {
+                    if (not defined $k) {
+                        croak "Unsupported hash key type 'undef' (step #$sc)";
+                    } elsif (ref $k) {
+                        croak "Unsupported hash key type '" . (ref $k) . "' (step #$sc)";
+                    } elsif ($k =~ /^[0-9a-zA-Z_]+$/) {
+                        # \w doesn't fit -- PPI can't parse unquoted utf8 hash keys
+                        # https://github.com/adamkennedy/PPI/issues/168#issuecomment-180506979
+                        push @items, $k;
+                    } else {
+                        push @items, map { $_ =~ s/'/\\'/g; "'$_'" } $k; # quote
+                    }
+                }
             } else {
                 croak "Unsupported hash definition (step #$sc)";
             }
@@ -229,7 +247,7 @@ L<Struct::Path>, L<Struct::Diff>, L<perldata>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2016 Michael Samoglyadov.
+Copyright 2016,2017 Michael Samoglyadov.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
